#include "CSocket.h"
#include <process.h>
#include <WinSock.h>
#pragma comment(lib, "WSock32.lib")

static unsigned __stdcall ThreadFunc( void *p )
{
	CSocket *pp = (CSocket*)p;
	return pp->Receive();
}

CSocket::CSocket():
	fTerminate(false)
{
	// Initialize Winsock DLL
	mWinsockErr = false;
	if( WSAStartup( MAKEWORD(2,0), &mWsaData ) != 0 )
	{
		mWinsockErr = true;
		return;
	}
}

CSocket::~CSocket()
{
	// Terminates Winsock DLL
	if( !mWinsockErr )
	{
		WSACleanup();
	}
	CloseHandle( mMutex );
	mMutex = NULL;
}

bool CSocket::Open( void* p )
{
	CONNECTION connetion = *(CONNECTION*)p;
    // Set the IP address and the port number of the server
	struct sockaddr_in sockaddr = {0};
	sockaddr.sin_family = AF_INET;									// Internet
	sockaddr.sin_addr.S_un.S_addr = inet_addr( connetion.ipAddr );	// IP address
	sockaddr.sin_port = htons( connetion.nPort );					// Port number

	// Create the TCP socket
	mSock = socket( AF_INET, SOCK_STREAM, 0 );
	if( mSock == INVALID_SOCKET )
	{
		return false;
	}

    // Establishes a connection to the TCP socket
	if( connect( mSock, (struct sockaddr *)&sockaddr, sizeof(sockaddr) ) == SOCKET_ERROR )
	{
		// Closes an existing socket
		closesocket( mSock );
		mSock = INVALID_SOCKET;
		return false;
	}

	// Create a recieve thread
	fTerminate = false;
	mHThread = (HANDLE)_beginthreadex(
		NULL,
		0,
		ThreadFunc,
		this,
		0,
		(unsigned int*)&mThreadId
	);

	if( mHThread == 0 )
	{
		// Closes an existing socket
		closesocket( mSock );
		mSock = INVALID_SOCKET;
		return false;
	}

	return true;
}

void CSocket::Close()
{
	fTerminate = true;

	// Disables sends or receives on a socket
	shutdown( mSock, 2 );

	// Retrieves the termination status of the thread
	DWORD dwExCode;
	GetExitCodeThread( mHThread , &dwExCode );
	if( dwExCode == STILL_ACTIVE )
	{
		// Wait until the thread is over
		WaitForSingleObject( mHThread, INFINITE );
		CloseHandle( mHThread );
	}

	// Closes an existing socket
	closesocket( mSock );
	mSock = INVALID_SOCKET;
}

DWORD CSocket::Receive()
{
	DWORD dwRead;
	BYTE buffer[RECV_BUFF_SIZE];

	while( !fTerminate )
	{
		// Read receive data
		dwRead = recv( mSock, (char*)buffer, sizeof(buffer), 0 );
		if( dwRead > 0 )
		{
			WriteBuffer( buffer, dwRead );
		}
	}

	return S_OK;
}

DWORD CSocket::Transmit( const BYTE* pBuf, DWORD dwSize )
{
	DWORD w, index = 0;

	while( dwSize > 0 )
	{
		w = send( mSock, (char*)pBuf + index, dwSize, 0 );
		index += w;
		if( !w )
		{
			return 0;
		}
		dwSize -= w;
	}

	return w;
}
